from scipy import sin, cos, sqrt, linspace, pi, dot
from Numeric import Float
from numpy import zeros,array
from gaussPivot import *
from analytical_prescription import *
from parameter import *
import os, time, csv, pprint
from domain_johns import *
from config import g, epsilon
from rootsearch import *
from bisect import *

#Analytical computations#################################################################
def root_g(a,b,t):
    dx = 0.01
    def g(u):
        return u + 2.0*A*pi/T*sin(2.0*pi/T*(t+u))
    while 1:
        x1,x2 = rootsearch(g,a,b,dx)
        if x1 != None:
            a = x2
            root = bisect(g,x1,x2,1)
        else:
            break
    return root
def shore(t):
    a = -1.0#-0.2#-1.0
    b = 1.0#0.2#1.0
    #dx = 0.01
    u = root_g(a,b,t)
    xi = -0.5*u*u + A*cos(2.0*pi/T*(t+u))
    position = 1.0 + xi
    return position, u




#Numerical computations###################################################################
def newtonRaphson2(f,q,tol=1.0e-15): ##1.0e-9 may be too large
    for i in range(30):                                                                                                                                    
        h = 1.0e-4      ##1.0e-4 may be too large.
        n = len(q)
        jac = zeros((n,n),Float)
        if 1.0+q[0]-x<0.0:
            temp1 = 1.0+q[0]-x
            q[0] = q[0]-temp1
            q[1] = v
            return q
        f0 = f(q)
        for i in range(n):
            temp = q[i]
            q[i] = temp + h
            f1 = f(q)
            q[i] = temp
            jac[:,i] = (f1 - f0)/h
        if sqrt(dot(f0,f0)/len(q)) < tol: return q
        dq = gaussPivot(jac,-f0)
        q = q + dq
        if sqrt(dot(dq,dq)) < tol*max(max(abs(q)),1.0): return q
    print 'Too many iterations'

def elevation(X):
    N = len(X)
    z = zeros(N,Float)
    for i in range(N):
        z[i] = (h_0/L)*X[i] - h_0
    return z

def height(X):
    N = len(X)
    z = zeros(N,Float)
    for i in range(N):
        z[i] = h_0 - (h_0/L)*X[i]
    return z

def velocity(X):
    N = len(X)
    return zeros(N,Float)
    
boundary = { (0,0): 'left',(N-1,1): 'right'}

domain = Domain(points,boundary)
domain.order = 2
domain.set_timestepping_method('rk2')
domain.set_CFL(1.0)
domain.beta = 1.0
domain.set_limiter("minmod")


def f_CG(t):
    timing = t*sqrt(g*h_0)/L
    w, u = prescribe(0.0,timing)
    wO = w*h_0
    uO = u*sqrt(g*h_0)
    zO = -h_0
    hO = wO - zO
    pO = uO * hO
    #[    'stage', 'xmomentum', 'elevation', 'height', 'velocity']
    return [wO,  pO,   zO,      hO,      uO]
def f_JOHNS(t):
    timing = t*sqrt(g*h_0)/L
    w, u = prescribe_at_O_JOHNS(timing)
    wO = w*h_0
    uO = u*sqrt(g*h_0)
    zO = -h_0
    hO = wO - zO
    pO = uO * hO
    #[    'stage', 'xmomentum', 'elevation', 'height', 'velocity']
    return [wO,  pO,   zO,      hO,      uO]

T1 = Time_boundary(domain,f_JOHNS)
D2 = Dirichlet_boundary([50.5,  0.0,  50.5,  0.0,  0.0])
domain.set_boundary({'left':T1,'right':D2})

domain.set_quantity('height',height)
domain.set_quantity('elevation',elevation)
domain.set_quantity('velocity',velocity)


Ver = domain.vertices
n_V = len(Ver)
AnalitW_V = zeros((n_V,2), Float)
AnalitP_V = zeros((n_V,2), Float)
AnalitZ_V = zeros((n_V,2), Float)
AnalitH_V = zeros((n_V,2), Float)
AnalitU_V = zeros((n_V,2), Float)

Cen = domain.centroids
n_C = len(Cen)
AnalitW_C = zeros(n_C, Float)
AnalitP_C = zeros(n_C, Float)
AnalitZ_C = zeros(n_C, Float)
AnalitH_C = zeros(n_C, Float)
AnalitU_C = zeros(n_C, Float)

waktu = 10.0     #3.0*60.0
WAKTU = 10.0#12690.0  #Note: Tp=15.0*60.0
yieldstep = finaltime = waktu
t0 = time.time()
counter=1

shorelines_numerical_johns = zeros(int(WAKTU/waktu), Float)
shorelines_analytical = zeros(int(WAKTU/waktu), Float)
time_instants = zeros(int(WAKTU/waktu), Float)
print "the initial time_instants=", time_instants


while finaltime < WAKTU+0.1:
    for t in domain.evolve(yieldstep = yieldstep, finaltime = finaltime):
        domain.write_time()
    time_instants[counter-1] = domain.time
    
    Stage = domain.quantities['stage']
    Momentum = domain.quantities['xmomentum']
    Elevation = domain.quantities['elevation']
    Height = domain.quantities['height']
    Velocity = domain.quantities['velocity']

    StageV = Stage.vertex_values
    MomV = Momentum.vertex_values
    ElevationV = Elevation.vertex_values
    HeightV = Height.vertex_values
    VelV = Velocity.vertex_values

    StageC = Stage.centroid_values
    MomC = Momentum.centroid_values
    ElevationC = Elevation.centroid_values
    HeightC = Height.centroid_values
    VelC = Velocity.centroid_values

    table = zeros((len(Ver.flat),6),Float)
    for r in range(len(Ver.flat)):
        for c in range(6):
            if c==0:
                table[r][c] = Ver.flat[r]
            elif c==1:
                table[r][c] = StageV.flat[r]
            elif c==2:
                table[r][c] = MomV.flat[r]
            elif c==3:
                table[r][c] = ElevationV.flat[r]
            elif c==4:
                table[r][c] = HeightV.flat[r]                
            else:
                table[r][c] = VelV.flat[r]

    outname = "%s%04i%s%f%s" %("numerical_johns_", counter, "_", domain.time, ".csv")                
    outfile = open(outname, 'w')
    writer = csv.writer(outfile)
    for row in table:
        writer.writerow(row)        
    outfile.close()    


    for s in range(2*n_V):
        heiL = HeightV.flat[s]
        momR = MomV.flat[s+1]
        if heiL >= 1e-6:
            if abs(momR)==0.0: #<1e-15:
                break
    #print "s+1=",s+1
    shorelines_numerical_johns[counter-1] = Ver.flat[s+1]
    #print "shorelines_numerical_johns=",shorelines_numerical_johns[counter-1]


    
    #print "Now the ANALYTIC"
    pos_shore, vel_shore = shore(domain.time*sqrt(g*h_0)/L) #dimensionless
    pos_shore = pos_shore*L
    vel_shore = vel_shore*sqrt(g*h_0)

    shorelines_analytical[counter-1] = pos_shore

    #The following is for calculating the error at centroids
    for i in range(n_C):
        x = Cen[i]
        if x < pos_shore:
            sta, vel = prescribe(x/L,domain.time*sqrt(g*h_0)/L) #dimensionless
            AnalitW_C[i] = sta*h_0
            AnalitU_C[i] = vel                            #It needs dimensionalisation
            AnalitZ_C[i] = (h_0/L)*x - h_0
            AnalitH_C[i] = AnalitW_C[i] - AnalitZ_C[i]
            AnalitP_C[i] = AnalitH_C[i]*AnalitU_C[i]  #It needs dimensionalisation
        else:
            AnalitW_C[i] = (h_0/L)*x - h_0                 
            AnalitU_C[i] = 0.0
            AnalitZ_C[i] = (h_0/L)*x - h_0
            AnalitH_C[i] = 0.0
            AnalitP_C[i] = 0.0
    AnalitU_C = AnalitU_C*sqrt(g*h_0) #This is the dimensionalisation
    AnalitP_C = AnalitP_C*sqrt(g*h_0) #This is the dimensionalisation

    error_W = (1.0/n_C)*sum(abs(StageC-AnalitW_C))
    error_P = (1.0/n_C)*sum(abs(MomC-AnalitP_C))
    error_U = (1.0/n_C)*sum(abs(VelC-AnalitU_C))
    table = zeros((1,4),Float)
    for r in range(1):
        for c in range(4):
            if c==0:
                table[r][c] = domain.time
            elif c==1:
                table[r][c] = error_W
            elif c==2:
                table[r][c] = error_P
            else:
                table[r][c] = error_U
    outname = "%s%04i%s%f%s" %("error_johns_", counter, "_", domain.time, ".csv")                
    outfile = open(outname, 'w')
    writer = csv.writer(outfile)
    for row in table:
        writer.writerow(row)        
    outfile.close()


    #The following is for ploting the quantities at vertex values
    for i in range(n_V):
        vector_x = Ver[i]
        for k in range(2):
            x = vector_x[k]
            if x < pos_shore:
                sta, vel = prescribe(x/L,domain.time*sqrt(g*h_0)/L) #dimensionless
                AnalitW_V[i,k] = sta*h_0
                AnalitU_V[i,k] = vel                            #It needs dimensionalisation
                AnalitZ_V[i,k] = (h_0/L)*x - h_0
                AnalitH_V[i,k] = AnalitW_V[i,k] - AnalitZ_V[i,k]
                AnalitP_V[i,k] = AnalitH_V[i,k]*AnalitU_V[i,k]  #It needs dimensionalisation
            else:
                AnalitW_V[i,k] = (h_0/L)*x - h_0                 
                AnalitU_V[i,k] = 0.0
                AnalitZ_V[i,k] = (h_0/L)*x - h_0
                AnalitH_V[i,k] = 0.0
                AnalitP_V[i,k] = 0.0
    AnalitU_V = AnalitU_V*sqrt(g*h_0) #This is the dimensionalisation
    AnalitP_V = AnalitP_V*sqrt(g*h_0) #This is the dimensionalisation


    table = zeros((len(Ver.flat),6),Float)
    for r in range(len(Ver.flat)):
        for c in range(6):
            if c==0:
                table[r][c] = Ver.flat[r]
            elif c==1:
                table[r][c] = AnalitW_V.flat[r]
            elif c==2:
                table[r][c] = AnalitP_V.flat[r]
            elif c==3:
                table[r][c] = AnalitZ_V.flat[r]
            elif c==4:
                table[r][c] = AnalitH_V.flat[r]                
            else:
                table[r][c] = AnalitU_V.flat[r]

    outname = "%s%04i%s%f%s" %("analytical_", counter, "_", domain.time, ".csv")                
    outfile = open(outname, 'w')
    writer = csv.writer(outfile)
    for row in table:
        writer.writerow(row)        
    outfile.close()

    #put this
    from pylab import clf,plot,title,xlabel,ylabel,legend,show,hold,subplot,ion#,savefig
    hold(False)
    clf()

    plot1 = subplot(311)
    plot(Ver.flat/1e+4,AnalitW_V.flat,'b-',  Ver.flat/1e+4,StageV.flat,'g-',  Ver.flat/1e+4,ElevationV.flat,'k-')
    #plot(Ver,StageV, Ver,ElevationV)
    #plot(Ver/L,StageV/h_0,  Ver/L,ElevationV/h_0)
    #xlabel('Position')
    ylabel('Stage')
    #plot1.set_xlim([0.0,1.2])
    plot1.set_ylim([-6.0,6.0])#([-9.0e-3,9.0e-3])
    #legend(('Analytical Solution', 'Numerical Solution', 'Discretized Bed'),
    #       'upper right', shadow=False)
    
    plot2 = subplot(312)
    plot(Ver.flat/1e+4,AnalitP_V.flat,'b-',  Ver.flat/1e+4,MomV.flat,'g-')
    #plot(Ver/L, VelV/sqrt(g*h_0))
    #xlabel('Position')
    ylabel('Momentum')
    #plot2.set_xlim([0.0,1.2])
    #legend(('Analytical Solution','Numerical Solution'),
    #       'upper right', shadow=False)        

    plot3 = subplot(313)
    plot(Ver.flat/1e+4,AnalitU_V.flat,'b-',  Ver.flat/1e+4,VelV.flat,'g-')
    #plot(Ver/L, VelV/sqrt(g*h_0))
    xlabel('Position / 10,000')
    ylabel('Velocity')
    #plot2.set_xlim([0.0,1.2])
    legend(('Analytical Solution','Numerical Solution'),
           'upper center', shadow=False)
    
    #filename = "%s%04i%s%f%s" %("numerical_johns_", counter,"_", domain.time, ".png")
    #savefig(filename)
    show()
    #raw_input("Press ENTER to continue")
    counter = counter+1
    finaltime = finaltime + waktu



table = zeros((int(WAKTU/waktu), 2),Float)
for r in range(int(WAKTU/waktu)):
    for c in range(2):
        if c==0:
            table[r][c] = time_instants[r]               
        else:
            table[r][c] = shorelines_numerical_johns[r]

outname = "%s" %("shore_numerical_johns.csv")                
outfile = open(outname, 'w')
writer = csv.writer(outfile)
for row in table:
    writer.writerow(row)        
outfile.close()


table = zeros((int(WAKTU/waktu), 2),Float)
for r in range(int(WAKTU/waktu)):
    for c in range(2):
        if c==0:
            table[r][c] = time_instants[r]               
        else:
            table[r][c] = shorelines_analytical[r]

outname = "%s" %("shore_analytical.csv")                
outfile = open(outname, 'w')
writer = csv.writer(outfile)
for row in table:
    writer.writerow(row)        
outfile.close()

